#!/usr/bin/env python

# picture_to_line_art.py
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
# 29/04/2023 Changed (mutilated?) by David Marsden to avoid error when original image is greyscale and
# commands updated where the previous ones are now deprecated.
#
# 10/06/2023 The option to select the paper colour added by David Marsden.
#
# 11/06/2023 Converts greyscale images to RGB so that paper colour works.
#
# 12/06/2023 Added ink colour and changed blend modes for paper and ink.
#			 Changed spline values to floating-point (with base 256 commented out in case required).
#			 Layers renamed.
#			 Pre-existing selection saved to channel for future use.
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release.
# Revision 1: 29/04/2023 - details as above.
# Revision 2: 10/06/2023 - details as above.
# Revision 3: 11/06/2023 - details as above.
# Revision 4: 12/06/2023 - details as above.
#
#
#import Image
from gimpfu import *
def python_picture_to_line_art(image, layer, pcolour, icolour) :
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	
	if not pdb.gimp_selection_is_empty(image):												# check if there is a selection. Note: double negative!
		old_select = pdb.gimp_selection_save(image)											# save pre-existing selection to channel	
	pdb.gimp_selection_none(image)															# remove any pre-existing selection
	
	base_type = pdb.gimp_image_base_type(image)												# check if RGB or GRAY
	if base_type == GRAY:																	# check image base type
		pdb.gimp_image_convert_rgb(image)													# if greyscale image convert to RGB

	layer_copy = pdb.gimp_layer_new_from_drawable(layer, image)
	pdb.gimp_image_insert_layer(image, layer_copy, None, 0)
	layer_name = "Desat"
	pdb.gimp_item_set_name(layer_copy, layer_name)
	
	base_type = pdb.gimp_image_base_type(image)
	if base_type != GRAY:																	# change - rev 1 -to work with greyscale images	
		pdb.gimp_drawable_desaturate(layer_copy, DESATURATE_LUMINANCE)						# (drawable, destination))
		
	
	#pdb.gimp_drawable_curves_spline(layer_copy,HISTOGRAM_VALUE,6,[0,0, 157,110, 255,255])	# does not work for everyone!
	
	pdb.gimp_drawable_curves_spline(layer_copy, HISTOGRAM_VALUE, 6, [0.0, 0.0, 0.615, 0.431, 1.0, 1.0])
	
	layer_copy2 = pdb.gimp_layer_new_from_drawable(layer_copy, image)						# new layer in dodge mode
	pdb.gimp_image_insert_layer(image, layer_copy2, None, 0)
	layer_name = "Dodge"
	pdb.gimp_item_set_name(layer_copy2, layer_name)	
	pdb.gimp_drawable_invert(layer_copy2, FALSE)	
	pdb.gimp_layer_set_mode(layer_copy2, DODGE_MODE)
	
	pdb.plug_in_gauss(image, layer_copy2, 10.0, 10.0, 1)									# blur the dodge layer
	 
	visible_layer = pdb.gimp_layer_new_from_visible(image, image, "visible")				# new layer in burn mode
	pdb.gimp_image_insert_layer(image, visible_layer, None, 0)
	layer_name = "Burn"
	pdb.gimp_item_set_name(visible_layer, layer_name)
	pdb.gimp_layer_set_mode(visible_layer, BURN_MODE)
	
	ink_layer = pdb.gimp_layer_new_from_visible(image, image, "visible")					# new layer for ink colour
	pdb.gimp_image_insert_layer(image, ink_layer, None, 0)
	layer_name = "Ink"
	pdb.gimp_item_set_name(ink_layer, layer_name)
	pdb.gimp_context_set_background(icolour)												# set background colour to that selected
	pdb.gimp_drawable_fill(ink_layer, FILL_BACKGROUND)										# add background colour to paper layer
	pdb.gimp_layer_set_mode(ink_layer, LAYER_MODE_LIGHTEN_ONLY)								# change layer mode to Lighten-Only
	pdb.gimp_layer_set_opacity(ink_layer, 90.0)												# set opacity at 90%
	
	paper_layer = pdb.gimp_layer_new_from_visible(image, image, "visible")					# new layer for paper colour
	pdb.gimp_image_insert_layer(image, paper_layer, None, 0)
	layer_name = "Paper"
	pdb.gimp_item_set_name(paper_layer, layer_name)
	pdb.gimp_context_set_foreground(pcolour)												# set foreground colour to that selected
	pdb.gimp_drawable_fill(paper_layer, FILL_FOREGROUND)									# add foreground colour to paper layer
	pdb.gimp_layer_set_mode(paper_layer, LAYER_MODE_DARKEN_ONLY)							# change layer mode to LCH Colour
	pdb.gimp_layer_set_opacity(paper_layer, 20.0)											# set opacity at 20%
		
	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()


register(
	"python_picture_to_line_art",
	"Picture To Line Art",
	"Picture To Line Art",
	"David Marsden",
	"David Marsden",
	"June 2023",
	"<Image>/Python-Fu/Picture To Line Art...",            					 			#Menu path
	"RGB*, GRAY*", 
	[
		(PF_COLOR, "pcolor", "Paper-colour", (255, 255, 255)),
		(PF_COLOR, "icolor", "Ink-colour", (0, 0, 0))
	],
	[],
	python_picture_to_line_art)

main()
